@@ -5,10 +5,7 @@ use crate::network::non_blocking_socket::{NonBlockingSocket, UdpNonBlockingSocke
 use crate::network::udp_msg::ConnectionStatus;
 use crate::network::udp_protocol::UdpProtocol;
 use crate::sync_layer::SyncLayer;
-use crate::{
-    Frame, GGRSEvent, GGRSRequest, PlayerHandle, PlayerType, SessionState, MAX_INPUT_BYTES,
-    MAX_PLAYERS, MAX_PREDICTION_FRAMES, NULL_FRAME,
-};
+use crate::{Frame, GGRSEvent, GGRSRequest, PlayerHandle, PlayerType, SessionState, NULL_FRAME};
 
 use std::collections::vec_deque::Drain;
 use std::collections::HashMap;
@@ -80,7 +77,7 @@ impl Player {
     }
 }
 
-#[derive(Debug, Copy, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub(crate) enum Event {
     /// The session is currently synchronizing with the remote client. It will continue until `count` reaches `total`.
     Synchronizing { total: u32, count: u32 },
@@ -103,6 +100,8 @@ pub struct P2PSession {
     num_players: u32,
     /// The number of bytes an input uses.
     input_size: usize,
+    /// The maximum number of frames GGRS will roll back. Every gamestate older than this is guaranteed to be correct.
+    max_prediction: usize,
     /// The sync layer handles player input queues and provides predictions.
     sync_layer: SyncLayer,
     /// FPS defines the expected update frequency of this session.
@@ -146,10 +145,11 @@ impl P2PSession {
     /// ```
     /// # use ggrs::{GGRSError, P2PSession};
     /// # fn main() -> Result<(), GGRSError> {
-    /// let local_port: u16 = 7777;
+    /// let local_port : u16 = 7777;
     /// let num_players : u32 = 2;
+    /// let max_pred : usize = 8;
     /// let input_size : usize = std::mem::size_of::<u32>();
-    /// let mut session = P2PSession::new(num_players, input_size, local_port)?;
+    /// let mut session = P2PSession::new(num_players, input_size, max_pred, local_port)?;
     /// # Ok(())
     /// # }
     /// ```
@@ -157,57 +157,53 @@ impl P2PSession {
     /// The created session will use the default socket type (currently UDP).
     ///
     /// # Errors
-    /// - Will return a `InvalidRequest` if the number of players is higher than the allowed maximum (see `MAX_PLAYERS`).
-    /// - Will return a `InvalidRequest` if `input_size` is higher than the allowed maximum (see `MAX_INPUT_BYTES`).
     /// - Will return `SocketCreationFailed` if the socket could not be created.
-    pub fn new(num_players: u32, input_size: usize, local_port: u16) -> Result<Self, GGRSError> {
+    pub fn new(
+        num_players: u32,
+        input_size: usize,
+        max_prediction: usize,
+        local_port: u16,
+    ) -> Result<Self, GGRSError> {
         // udp nonblocking socket creation
         let addr = SocketAddr::new(IpAddr::V4(Ipv4Addr::UNSPECIFIED), local_port); //TODO: IpV6?
         let socket =
             Box::new(UdpNonBlockingSocket::new(addr).map_err(|_| GGRSError::SocketCreationFailed)?);
-        Self::new_impl(num_players, input_size, socket)
+        Ok(Self::new_impl(
+            num_players,
+            input_size,
+            max_prediction,
+            socket,
+        ))
     }
 
     /// Creates a new `P2PSession` for players who participate on the game input. After creating the session, add local and remote players,
     /// set input delay for local players and then start the session. The session will use the provided socket.
-    ///
-    /// # Errors
-    /// - Will return a `InvalidRequest` if the number of players is higher than the allowed maximum (see `MAX_PLAYERS`).
-    /// - Will return a `InvalidRequest` if `input_size` is higher than the allowed maximum (see `MAX_INPUT_BYTES`).
     pub fn new_with_socket(
         num_players: u32,
         input_size: usize,
+        max_prediction: usize,
         socket: impl NonBlockingSocket + 'static,
-    ) -> Result<Self, GGRSError> {
-        Self::new_impl(num_players, input_size, Box::new(socket))
+    ) -> Self {
+        Self::new_impl(num_players, input_size, max_prediction, Box::new(socket))
     }
 
     fn new_impl(
         num_players: u32,
         input_size: usize,
+        max_prediction: usize,
         socket: Box<dyn NonBlockingSocket>,
-    ) -> Result<Self, GGRSError> {
-        if num_players > MAX_PLAYERS {
-            return Err(GGRSError::InvalidRequest {
-                info: "Too many players.".to_owned(),
-            });
-        }
-        if input_size > MAX_INPUT_BYTES {
-            return Err(GGRSError::InvalidRequest {
-                info: "Input size too big.".to_owned(),
-            });
-        }
-
+    ) -> Self {
         // local connection status
         let mut local_connect_status = Vec::new();
         for _ in 0..num_players {
             local_connect_status.push(ConnectionStatus::default());
         }
 
-        Ok(Self {
+        Self {
             state: SessionState::Initializing,
             num_players,
             input_size,
+            max_prediction,
             fps: DEFAULT_FPS,
             sparse_saving: DEFAULT_SAVE_MODE,
             socket,
@@ -215,13 +211,13 @@ impl P2PSession {
             next_recommended_sleep: 0,
             next_spectator_frame: 0,
             frames_ahead: 0,
-            sync_layer: SyncLayer::new(num_players, input_size),
+            sync_layer: SyncLayer::new(num_players, input_size, max_prediction),
             disconnect_timeout: DEFAULT_DISCONNECT_TIMEOUT,
             disconnect_notify_start: DEFAULT_DISCONNECT_NOTIFY_START,
             disconnect_frame: NULL_FRAME,
             players: HashMap::new(),
             event_queue: VecDeque::new(),
-        })
+        }
     }
 
     /// Must be called for each player in the session (e.g. in a 3 player session, must be called 3 times) before starting the session. Returns the player handle
@@ -284,6 +280,18 @@ impl P2PSession {
         {
             endpoint.synchronize();
         }
+
+        // if there are no remote players, then run
+        if self
+            .players
+            .values()
+            .filter_map(Player::as_endpoint)
+            .count()
+            == 0
+        {
+            self.state = SessionState::Running;
+        }
+
         Ok(())
     }
 
@@ -372,7 +380,7 @@ impl P2PSession {
         // in sparse saving mode, we need to make sure not to lose the last saved frame
         let last_saved = self.sync_layer.last_saved_frame();
         if self.sparse_saving
-            && self.sync_layer.current_frame() - last_saved >= MAX_PREDICTION_FRAMES as i32
+            && self.sync_layer.current_frame() - last_saved >= self.max_prediction as i32
         {
             // check if the current frame is confirmed, otherwise we need to roll back
             if confirmed_frame >= self.sync_layer.current_frame() {
@@ -412,14 +420,16 @@ impl P2PSession {
         }
 
         //create an input struct for current frame
-        let mut game_input: GameInput =
-            GameInput::new(self.sync_layer.current_frame(), self.input_size);
-        game_input.copy_input(local_input);
+        let mut game_input: GameInput = GameInput::new(
+            self.sync_layer.current_frame(),
+            self.input_size,
+            local_input.to_vec(),
+        );
 
         // send the input into the sync layer
         let actual_frame = self
             .sync_layer
-            .add_local_input(local_player_handle, game_input)?;
+            .add_local_input(local_player_handle, game_input.clone())?;
 
         // if the actual frame is the null frame, the frame has been dropped by the input queues (for example due to changed input delay)
         if actual_frame != NULL_FRAME {
@@ -433,7 +443,7 @@ impl P2PSession {
                 .filter_map(Player::remote_as_endpoint_mut)
             {
                 // send the input directly
-                endpoint.send_input(game_input, &self.local_connect_status);
+                endpoint.send_input(game_input.clone(), &self.local_connect_status);
                 endpoint.send_all_messages(&mut self.socket);
             }
         }
@@ -634,6 +644,30 @@ impl P2PSession {
         Ok(())
     }
 
+    /// Returns the highest confirmed frame. We have received all input for this frame and it is thus correct.
+    pub fn confirmed_frame(&self) -> Frame {
+        let mut confirmed_frame = i32::MAX;
+
+        for con_stat in &self.local_connect_status {
+            if !con_stat.disconnected {
+                confirmed_frame = std::cmp::min(confirmed_frame, con_stat.last_frame);
+            }
+        }
+
+        assert!(confirmed_frame < i32::MAX);
+        confirmed_frame
+    }
+
+    /// Returns the current frame of a session.
+    pub const fn current_frame(&self) -> Frame {
+        self.sync_layer.current_frame()
+    }
+
+    /// Returns the maximum prediction window of a session.
+    pub const fn max_prediction(&self) -> usize {
+        self.max_prediction
+    }
+
     /// Returns the handle of the local player, if the player is already added
     pub fn local_player_handle(&self) -> Option<PlayerHandle> {
         self.players
@@ -861,23 +895,25 @@ impl P2PSession {
         }
 
         while self.next_spectator_frame <= confirmed_frame {
-            let inputs = self
+            let mut inputs = self
                 .sync_layer
                 .confirmed_inputs(self.next_spectator_frame, &self.local_connect_status);
             assert_eq!(inputs.len(), self.num_players as usize);
+
             // construct a pseudo input containing input of all players for the spectators
-            let mut spectator_input = GameInput::new(
-                self.next_spectator_frame,
-                self.input_size * self.num_players as usize,
-            );
-            for (i, input) in inputs.iter().enumerate() {
+            let mut concatenated_buffer = Vec::new();
+            for input in inputs.iter_mut() {
                 assert!(input.frame == NULL_FRAME || input.frame == self.next_spectator_frame);
                 assert!(input.frame == NULL_FRAME || input.size == self.input_size);
-                let start = i * input.size;
-                let end = (i + 1) * input.size;
-                spectator_input.buffer[start..end].copy_from_slice(input.input());
+                concatenated_buffer.append(&mut input.buffer);
             }
 
+            let spectator_input = GameInput::new(
+                self.next_spectator_frame,
+                self.input_size * self.num_players as usize,
+                concatenated_buffer,
+            );
+
             // send it off
             for endpoint in self
                 .players
@@ -885,7 +921,7 @@ impl P2PSession {
                 .filter_map(Player::spectator_as_endpoint_mut)
             {
                 if endpoint.is_running() {
-                    endpoint.send_input(spectator_input, &self.local_connect_status);
+                    endpoint.send_input(spectator_input.clone(), &self.local_connect_status);
                 }
             }
 
@@ -894,20 +930,6 @@ impl P2PSession {
         }
     }
 
-    /// Returns the confirmed frame. We have received all input for this frame and it is thus correct.
-    fn confirmed_frame(&mut self) -> Frame {
-        let mut confirmed_frame = i32::MAX;
-
-        for con_stat in &self.local_connect_status {
-            if !con_stat.disconnected {
-                confirmed_frame = std::cmp::min(confirmed_frame, con_stat.last_frame);
-            }
-        }
-
-        assert!(confirmed_frame < i32::MAX);
-        confirmed_frame
-    }
-
     /// Check if players are registered as disconnected for earlier frames on other remote players in comparison to our local assumption.
     /// Disconnect players that are disconnected for other players and update the frame they disconnected
     fn update_player_disconnects(&mut self) {
