@@ -4,9 +4,7 @@ use crate::error::GGRSError;
 use crate::frame_info::GameInput;
 use crate::network::udp_msg::ConnectionStatus;
 use crate::sync_layer::SyncLayer;
-use crate::{
-    Frame, GGRSRequest, PlayerHandle, MAX_INPUT_BYTES, MAX_PLAYERS, MAX_PREDICTION_FRAMES,
-};
+use crate::{Frame, GGRSRequest, PlayerHandle};
 
 /// During a `SyncTestSession`, GGRS will simulate a rollback every frame and resimulate the last n states, where n is the given check distance.
 /// The resimulated checksums will be compared with the original checksums and report if there was a mismatch.
@@ -14,7 +12,8 @@ use crate::{
 pub struct SyncTestSession {
     num_players: u32,
     input_size: usize,
-    check_distance: u32,
+    max_prediction: usize,
+    check_distance: usize,
     sync_layer: SyncLayer,
     dummy_connect_status: Vec<ConnectionStatus>,
     checksum_history: HashMap<Frame, u64>,
@@ -31,43 +30,43 @@ impl SyncTestSession {
     /// ```
     /// # use ggrs::{GGRSError, SyncTestSession};
     /// # fn main() -> Result<(), GGRSError> {
-    /// let check_distance : u32 = 7;
+    /// let check_distance : usize = 7;
+    /// let max_pred : usize = 8;
     /// let num_players : u32 = 2;
     /// let input_size : usize = std::mem::size_of::<u32>();
-    /// let mut session = SyncTestSession::new(num_players, input_size, check_distance)?;
+    /// let mut session = SyncTestSession::new(num_players, input_size, max_pred, check_distance)?;
     /// # Ok(())
     /// # }
     /// ```
     ///
     /// # Errors
-    /// - Will return a `InvalidRequestError` if the number of players is higher than the allowed maximum (see `MAX_PLAYERS`).
-    /// - Will return a `InvalidRequestError` if `input_size` is higher than the allowed maximum (see `MAX_INPUT_BYTES`).
     /// - Will return a `InvalidRequestError` if the `check_distance is` higher than or equal to `MAX_PREDICTION_FRAMES`.
     pub fn new(
         num_players: u32,
         input_size: usize,
-        check_distance: u32,
+        max_prediction: usize,
+        check_distance: usize,
     ) -> Result<Self, GGRSError> {
-        if num_players > MAX_PLAYERS {
-            return Err(GGRSError::InvalidRequest {
-                info: "Too many players.".to_owned(),
-            });
-        }
-        if input_size > MAX_INPUT_BYTES {
-            return Err(GGRSError::InvalidRequest {
-                info: "Input size too big.".to_owned(),
-            });
-        }
-        if check_distance >= MAX_PREDICTION_FRAMES {
+        if check_distance >= max_prediction {
             return Err(GGRSError::InvalidRequest {
                 info: "Check distance too big.".to_owned(),
             });
         }
-        Ok(Self::new_impl(num_players, input_size, check_distance))
+        Ok(Self::new_impl(
+            num_players,
+            input_size,
+            max_prediction,
+            check_distance,
+        ))
     }
 
     /// Creates a new `SyncTestSession` instance with given values.
-    fn new_impl(num_players: u32, input_size: usize, check_distance: u32) -> Self {
+    fn new_impl(
+        num_players: u32,
+        input_size: usize,
+        max_prediction: usize,
+        check_distance: usize,
+    ) -> Self {
         let mut dummy_connect_status = Vec::new();
         for _ in 0..num_players {
             dummy_connect_status.push(ConnectionStatus::default());
@@ -75,8 +74,9 @@ impl SyncTestSession {
         Self {
             num_players,
             input_size,
+            max_prediction,
             check_distance,
-            sync_layer: SyncLayer::new(num_players, input_size),
+            sync_layer: SyncLayer::new(num_players, input_size, max_prediction),
             dummy_connect_status,
             checksum_history: HashMap::default(),
         }
@@ -112,9 +112,11 @@ impl SyncTestSession {
         assert_eq!(self.num_players as usize, all_inputs.len());
         for (i, input_bytes) in all_inputs.iter().enumerate() {
             //create an input struct for current frame
-            let mut input: GameInput =
-                GameInput::new(self.sync_layer.current_frame(), self.input_size);
-            input.copy_input(input_bytes);
+            let input: GameInput = GameInput::new(
+                self.sync_layer.current_frame(),
+                self.input_size,
+                input_bytes.to_vec(),
+            );
 
             // send the input into the sync layer
             self.sync_layer.add_local_input(i, input)?;
@@ -179,6 +181,11 @@ impl SyncTestSession {
         self.input_size
     }
 
+    /// Returns the maximum prediction window of a session.
+    pub const fn max_prediction(&self) -> usize {
+        self.max_prediction
+    }
+
     /// Updates the `checksum_history` and checks if the checksum is identical if it already has been recorded once
     fn checksums_consistent(&mut self, frame_to_check: Frame) -> bool {
         // remove entries older than the check_distance
