@@ -8,13 +8,13 @@ mod stubs;
 #[test]
 #[serial]
 fn test_create_session() {
-    assert!(P2PSession::new(2, stubs::INPUT_SIZE, 7777).is_ok());
+    assert!(P2PSession::new(2, stubs::INPUT_SIZE, stubs::MAX_PRED_FRAMES, 7777).is_ok());
 }
 
 #[test]
 #[serial]
 fn test_add_player() {
-    let mut sess = P2PSession::new(2, stubs::INPUT_SIZE, 7777).unwrap();
+    let mut sess = P2PSession::new(2, stubs::INPUT_SIZE, stubs::MAX_PRED_FRAMES, 7777).unwrap();
     let addr = SocketAddr::new(IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)), 8080);
     assert!(sess.add_player(PlayerType::Local, 0).is_ok());
     assert!(sess.add_player(PlayerType::Remote(addr), 1).is_ok());
@@ -29,7 +29,7 @@ fn test_add_player() {
 #[test]
 #[serial]
 fn test_start_session() {
-    let mut sess = P2PSession::new(2, stubs::INPUT_SIZE, 7777).unwrap();
+    let mut sess = P2PSession::new(2, stubs::INPUT_SIZE, stubs::MAX_PRED_FRAMES, 7777).unwrap();
     let addr = SocketAddr::new(IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)), 8080);
     assert!(sess.add_player(ggrs::PlayerType::Local, 0).is_ok());
     assert!(sess.start_session().is_err()); // not enough players
@@ -41,7 +41,7 @@ fn test_start_session() {
 #[test]
 #[serial]
 fn test_disconnect_player() {
-    let mut sess = P2PSession::new(2, stubs::INPUT_SIZE, 7777).unwrap();
+    let mut sess = P2PSession::new(2, stubs::INPUT_SIZE, stubs::MAX_PRED_FRAMES, 7777).unwrap();
     let addr = SocketAddr::new(IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)), 8080);
     assert!(sess.add_player(ggrs::PlayerType::Local, 0).is_ok());
     assert!(sess.add_player(ggrs::PlayerType::Remote(addr), 1).is_ok());
@@ -54,8 +54,8 @@ fn test_disconnect_player() {
 #[test]
 #[serial]
 fn test_synchronize_p2p_sessions() {
-    let mut sess1 = P2PSession::new(2, stubs::INPUT_SIZE, 7777).unwrap();
-    let mut sess2 = P2PSession::new(2, stubs::INPUT_SIZE, 8888).unwrap();
+    let mut sess1 = P2PSession::new(2, stubs::INPUT_SIZE, stubs::MAX_PRED_FRAMES, 7777).unwrap();
+    let mut sess2 = P2PSession::new(2, stubs::INPUT_SIZE, stubs::MAX_PRED_FRAMES, 8888).unwrap();
     let addr1 = SocketAddr::new(IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)), 7777);
     let addr2 = SocketAddr::new(IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)), 8888);
 
@@ -87,8 +87,8 @@ fn test_synchronize_p2p_sessions() {
 fn test_advance_frame_p2p_sessions() {
     let mut stub1 = stubs::GameStub::new();
     let mut stub2 = stubs::GameStub::new();
-    let mut sess1 = P2PSession::new(2, stubs::INPUT_SIZE, 7777).unwrap();
-    let mut sess2 = P2PSession::new(2, stubs::INPUT_SIZE, 8888).unwrap();
+    let mut sess1 = P2PSession::new(2, stubs::INPUT_SIZE, stubs::MAX_PRED_FRAMES, 7777).unwrap();
+    let mut sess2 = P2PSession::new(2, stubs::INPUT_SIZE, stubs::MAX_PRED_FRAMES, 8888).unwrap();
     let addr1 = SocketAddr::new(IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)), 7777);
     let addr2 = SocketAddr::new(IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)), 8888);
 
